package com.wanji.indicators.task.track.service;

import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.SingleCarTrackListModel;
import com.wanji.indicators.task.track.service.func.ProcessCarTrackByKey;
import com.wanji.indicators.task.track.service.key.KeySelectorGlobalIdPlateNo;
import com.wanji.indicators.task.track.service.sink.BatchTrackElasticSearchSinkFunction;
import com.wanji.indicators.task.track.service.sink.PartitionBatchTrackElasticSearchSinkFunction;
import com.wanji.indicators.util.ElasticSearchSinkUtil;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.windowing.assigners.TumblingEventTimeWindows;
import org.apache.flink.streaming.api.windowing.time.Time;
import org.apache.http.HttpHost;

import java.io.Serializable;
import java.net.MalformedURLException;
import java.time.Duration;
import java.util.List;
import java.util.Properties;

public class TrackStoreMainNew   {
    private final StreamExecutionEnvironment env;

    private final PropertiesHelper instance;

    private Properties p = new Properties();

    private String path;

    private String indexName ;
    //窗口时间，单位秒
    private Integer windowTime=5;


    private TrackStoreMainNew(StreamExecutionEnvironment env,String paramInfo) {
        this.env = env;
        String[] sps = paramInfo.split(",");
        this.indexName = sps[0];
        if (sps.length>1)
            this.windowTime = Integer.parseInt(sps[1]);
        this.instance = PropertiesHelper.getInstance();
        this.p = this.instance.getProperties();
    }


    public static TrackStoreMainNew init(StreamExecutionEnvironment env,String paramInfo) {
        return new TrackStoreMainNew(env,paramInfo);
    }


    public void run(SingleOutputStreamOperator<CarTrackModel> carTrackModelStream) {

        //根据车辆上报时间每5分钟一个窗口累积
        SingleOutputStreamOperator<SingleCarTrackListModel> groupByPlateStream =
                carTrackModelStream.assignTimestampsAndWatermarks(
                        WatermarkStrategy.<CarTrackModel>forBoundedOutOfOrderness(Duration.ofSeconds(1))
                                .withTimestampAssigner((event, timestamp) -> event.getGlobalTimeStamp()))
                        .keyBy(new KeySelectorGlobalIdPlateNo())
                        .window(TumblingEventTimeWindows.of(Time.seconds(this.windowTime)))
                        //.trigger(new CountTriggerWithTimeout<>(Integer.MAX_VALUE,TimeCharacteristic.EventTime))
                        .process(new ProcessCarTrackByKey())
                        .setParallelism(2)
                        .name("车辆轨迹分组-（全域ID-车辆ID-车牌号）");


        //从配置文件中读取 es 的地址
        try {
            List<HttpHost> esAddresses = ElasticSearchSinkUtil.getEsAddresses(p.getProperty("elasticsearch.server"));

            BatchTrackElasticSearchSinkFunction elasticsearchSinkFunction = new BatchTrackElasticSearchSinkFunction(indexName);
            ElasticSearchSinkUtil.addSink(esAddresses, 2, groupByPlateStream, elasticsearchSinkFunction);


            PartitionBatchTrackElasticSearchSinkFunction elasticsearchSinkFunction1 = new PartitionBatchTrackElasticSearchSinkFunction(indexName);
            ElasticSearchSinkUtil.addSink(esAddresses, 2, groupByPlateStream, elasticsearchSinkFunction1);


        } catch (MalformedURLException e) {
            e.printStackTrace();
        }
    }


}
