package com.wanji.indicators.task.track.stream;

import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.FrameModel;
import com.wanji.indicators.task.track.service.AreaRealTimeIndicatorMainNew;
import com.wanji.indicators.task.track.service.CarRoutePathMainNew;
import com.wanji.indicators.task.track.service.TrackStoreMainNew;
import com.wanji.indicators.task.track.service.func.CarTrackFlatMap;
import com.wanji.indicators.task.track.service.func.FrameFlatMap;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
import org.apache.flink.connector.kafka.source.reader.deserializer.KafkaRecordDeserializationSchema;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.kafka.clients.consumer.OffsetResetStrategy;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.joda.time.DateTime;

import java.util.Properties;

/**
 * 对单辆车轨迹，每30秒封装一个批次
 */
public class TrackUnionMain {
  //private static final Logger log = LoggerFactory.getLogger(TrafficEventMain.class);

  public static void main(String[] args) {
    //批量轨迹es index,逗号分割：esIndexName,windowTime
    String indexName = "";//args[0];

    PropertiesHelper instance = PropertiesHelper.getInstance();
    Properties properties = instance.getProperties();

    StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
    env.setParallelism(1);

    DateTime currentDateTime = new DateTime();
    //过去一天0点
    DateTime dateTime = currentDateTime.withMillisOfDay(0).plusDays(-1);

    KafkaSource<String> source = KafkaSource.<String>builder()
             .setProperties(instance.getConsumerProperties())
            .setProperty("auto.offset.commit", "true")
            .setProperty("auto.commit.interval.ms","1000")
            .setProperty("commit.offsets.on.checkpoint","true")
            .setBootstrapServers(properties.getProperty("bootstrap.servers"))
            .setTopics(properties.getProperty("consumer.topic"))
            .setGroupId(properties.getProperty("consumer.group.id")+"_BATCH_TRACK")
            /* 设置起始偏移量有以下几种情况：
                   1.从指定的位置消费：OffsetsInitializer.offsets(Map< TopicPartition, Long> offsets)
                   2.从最新位置消费(最后一条处)：OffsetsInitializer.latest()
                   3.从最早位置消费(第一条处)：OffsetsInitializer.earliest()
                   4.从上次提交的位置消费：OffsetsInitializer.committedOffsets()
                   5.新的组，从来没有提交过,再指定一个消费方式：OffsetsInitializer.committedOffsets(OffsetResetStrategy.LATEST)
                  */
            .setStartingOffsets(OffsetsInitializer.committedOffsets(OffsetResetStrategy.LATEST))//设置起始偏移量，也就是从哪里消费
            // 从大于等于此时间戳开始的偏移量开始
            //.setStartingOffsets(OffsetsInitializer.timestamp(dateTime.getMillis()))

            .setDeserializer(KafkaRecordDeserializationSchema.valueOnly(StringDeserializer.class))

            .build();

    DataStream<String> stream = env
            .fromSource(source, WatermarkStrategy.noWatermarks(),"kafka-car-track-source");

    SingleOutputStreamOperator<FrameModel> frameModelStream =
            stream
                    .flatMap(new FrameFlatMap())
                    .setParallelism(1)
                    .name("轨迹帧数据-JsonToObject");

    SingleOutputStreamOperator<CarTrackModel> carTrackModelStream =
            frameModelStream.
                    flatMap(new CarTrackFlatMap())
                    .setParallelism(1)
                    .name("轨迹帧数据解析-ToCarTrackModel");

    try {

      //批量轨迹封装
      TrackStoreMainNew.init(env,indexName).run(carTrackModelStream);

      //AreaRealTimeIndicatorMainNew.init(env,indexName).run(carTrackModelStream);

      env.execute("轨迹处理工作流");

    } catch (Exception e) {
      e.printStackTrace();
      //log.error("交通指标计算任务异常 :   " + e);
    }
  }
}
