package com.wanji.indicators.task.travelTime.service;

import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.config.RedisConfig;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.task.queuelength.service.func.QueueAggregateFunction;
import com.wanji.indicators.task.queuelength.service.func.QueueProcessWindow;
import com.wanji.indicators.task.queuelength.service.model.MaxQueueResultModel;
import com.wanji.indicators.task.queuelength.service.sink.QueueRedisMapper;
import com.wanji.indicators.task.track.service.key.KeySelectorGlobalIdPlateNo;
import com.wanji.indicators.task.track.service.sink.BatchTrackElasticSearchSinkFunction;
import com.wanji.indicators.task.travelTime.service.func.TravelAggregateFunction;
import com.wanji.indicators.task.travelTime.service.func.TravelProcessWindow;
import com.wanji.indicators.task.travelTime.service.model.TravelTimeResultModel;
import com.wanji.indicators.task.travelTime.service.sink.TravelRedisMapper;
import com.wanji.indicators.util.ElasticSearchSinkUtil;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.java.tuple.Tuple3;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.streaming.api.windowing.assigners.EventTimeSessionWindows;
import org.apache.flink.streaming.api.windowing.assigners.SlidingEventTimeWindows;
import org.apache.flink.streaming.api.windowing.time.Time;
import org.apache.flink.streaming.connectors.redis.RedisSink;
import org.apache.flink.util.Collector;
import org.apache.http.HttpHost;

import java.io.Serializable;
import java.net.MalformedURLException;
import java.time.Duration;
import java.util.List;
import java.util.Properties;

public class MaxTravelTimeMainNew implements Serializable {
    private final StreamExecutionEnvironment env;

    private final PropertiesHelper instance;

    private Properties p = new Properties();

    private String path;

    private String indexName = "area.realtime.indicator";

    private int windowTime = 2;

    private MaxTravelTimeMainNew(StreamExecutionEnvironment env, String paramInfo) {
        this.env = env;
        this.instance = PropertiesHelper.getInstance();
        this.p = this.instance.getProperties();
    }


    public static MaxTravelTimeMainNew init(StreamExecutionEnvironment env, String paramInfo) {
        return new MaxTravelTimeMainNew(env,paramInfo);
    }


    public void run(SingleOutputStreamOperator<CarTrackModel> filterStream) {

        SingleOutputStreamOperator<TravelTimeResultModel> groupByPlateStream =
                filterStream.assignTimestampsAndWatermarks(
                        WatermarkStrategy.<CarTrackModel>forBoundedOutOfOrderness(Duration.ofSeconds(5))
                                .withTimestampAssigner((event, timestamp) -> event.getGlobalTimeStamp()))
                        .keyBy(new KeySelectorGlobalIdPlateNo())
                        .window(EventTimeSessionWindows.withGap(Time.minutes(2)))
                        .aggregate(new TravelAggregateFunction(),new TravelProcessWindow())
                        .name("旅行时间、速度计算");


        try {
            SingleOutputStreamOperator<TravelTimeResultModel>  travelTimeStream = groupByPlateStream.keyBy(TravelTimeResultModel::getId).countWindow(2).maxBy("travelTime");
            travelTimeStream.print();
            travelTimeStream.addSink( new RedisSink<TravelTimeResultModel>(new RedisConfig().getRedisConfig(), new TravelRedisMapper(Constant.AREA_REALTIME_INDICATOR)));

//            SingleOutputStreamOperator<TravelTimeResultModel>  travelSpeedStream = groupByPlateStream.keyBy(TravelTimeResultModel::getId).countWindow(2).maxBy("travelSpeed");
//            travelTimeStream.addSink( new RedisSink<TravelTimeResultModel>(new RedisConfig().getRedisConfig(), new TravelRedisMapper(Constant.AREA_REALTIME_INDICATOR)));

        } catch (Exception e) {
            e.printStackTrace();
        }
    }


}
